(define-module (ahttpd alterator)
  :use-module (vhttpd)
  :use-module (ahttpd response)
  :use-module (alterator http utils)
  :use-module (alterator http locale)
  :use-module (alterator http html)
  :use-module (alterator http template)
  :export (make-alterator-response))

(define *xml-entities* '("amp" "apos" "quot" "lt" "gt"))

(define (make-langlist lang cookie)
  (map code->locale (or (and cookie (list cookie))
                        (decode-languages lang)
                        '("en-us"))))

(define (scm->string scm)
  (with-output-to-string
    (lambda() (scm->out scm display))))

(define (replace-local-refs)
  (replace-tag: "local:a"
		(lambda(options content)
		  `(a ,@options ,@content))))

(define (post-process/html scm)
  (scm-filter scm
	      (make-cb
		(replace-local-refs))))

(define (post-process/xml scm)
  (scm-filter
    scm
    (make-cb
      (replace-tag: "&" ;;remove non-xml entities
		    (lambda(options content)
		      (if (memq (car content) *xml-entities*)
			`(& ,(car content))
			"")))
      (replace-tag: "input"
		    (@ 'type "submit") "")
      (replace-tag: "input"
		    (@ 'type "reset") "")
      (replace-local-refs))))

(define (redirect? answer)
  (and (pair? answer) (eq? (car answer) 'redirect)))

(define (async? args)
   (let ((v (assoc "async" args)))
    (and (pair? v) (string=? (cdr v) "yes"))))

(define (socket-write name request)
  (let ((s (socket PF_UNIX SOCK_STREAM 0)))
    (connect s AF_UNIX name)
    (write request s)
    (let ((answer (read s)))
      (close-port s)
      answer)))

(define (make-args request)
  (acons "language"
	 (make-langlist (message-header request "accept-language")
			(message-cookie request "language"))
	 (append (decode-content (message-header request "content-type")
				 (message-body request))
		 (decode-url (message-query request)))))

(define (make-alterator-response socket-name uri request)
  (let* ((method (message-method request))
	 (args   (make-args request))
	 (answer (socket-write socket-name (list method uri args))))
    (cond
      ((redirect? answer)
       (make-redirect-response (cadr answer)))
      ((async? args)
       (make-xml-response 200 (scm->string (post-process/xml answer))))
      (else
	(make-html-response 200 (scm->string (post-process/html answer)))))))
