(define-module (ahttpd auth basic)
  :use-module (srfi srfi-2)
  :use-module (srfi srfi-13)
  :use-module (vhttpd)
  :export (basic-auth-check
	   basic-auth-challenge))

(load-extension "libguile-ahttpd.so" "scm_init_ahttpd")

;;; users cache

(define *users* '())

(define (remember-user name password)
  (set! *users* (acons name password *users*))
  name)

(define (find-user name password)
  (cond
    ((assoc name *users*)
     =>
     (lambda(x) (and (string=? (cdr x) password) name)))
    (else #f)))

;;; header parsing

(define (split-creds str)
  (and-let* ((str (decode-base64 str))
	     (idx (string-index str #\:)))
   (cons
     (substring str 0 idx)
     (substring str (+ idx 1)))))

(define (split-header request)
  (and-let* ((header (message-header request "authorization"))
	     (data   (string-tokenize header))
	     (scheme (car data))
	     (creds  (and (pair? (cdr data)) (cadr data))))
	    (cons scheme creds)))

(define (basic-auth-check request)
  (and-let* ((header (split-header request))
	     ((string=? (car header) "Basic"))
	     (creds (split-creds (cdr header))))
	    (or (find-user (car creds) (cdr creds))
		(and (check-password (car creds) (cdr creds))
		     (remember-user (car creds) (cdr creds))))))

(define (basic-auth-challenge realm)
  (let ((response (make-response 401)))
    (message-header-set! response "www-authenticate" (format #f "Basic realm=\"~A\"" realm))
    response))
